@@ -59,6 +59,8 @@
 #include <libgnome/gnome-desktop-item.h>
 #include <libgnome/gnome-desktop-utils.h>
 
+#include "private.h"
+
 struct _GnomeDesktopItem {
 	int refcount;
 
@@ -347,19 +349,6 @@ type_from_string (const char *type)
 	return GNOME_DESKTOP_ITEM_TYPE_OTHER;
 }
 
-static void
-init_i18n (void) {
-	static gboolean initialized = FALSE;
-	
-	if (!initialized) {
-		bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
-#ifdef HAVE_BIND_TEXTDOMAIN_CODESET
-		bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-#endif
-		initialized = TRUE;
-	}
-}
-
 /**
  * gnome_desktop_item_new:
  *
@@ -372,7 +361,7 @@ gnome_desktop_item_new (void)
 {
 	GnomeDesktopItem *retval;
 
-	init_i18n ();
+	_gnome_desktop_init_i18n ();
 
 	retval = g_new0 (GnomeDesktopItem, 1);
 
@@ -385,6 +374,9 @@ gnome_desktop_item_new (void)
 	/* These are guaranteed to be set */
 	gnome_desktop_item_set_string (retval,
 				       GNOME_DESKTOP_ITEM_NAME,
+				       /* Translators: the "name" mentioned
+					* here is the name of an application or
+					* a document */
 				       _("No name"));
 	gnome_desktop_item_set_string (retval,
 				       GNOME_DESKTOP_ITEM_ENCODING,
@@ -796,7 +788,7 @@ gnome_desktop_item_new_from_basename (const char *basename,
 		g_set_error (error,
 			     GNOME_DESKTOP_ITEM_ERROR,
 			     GNOME_DESKTOP_ITEM_ERROR_CANNOT_OPEN,
-			     _("Error cannot find file id '%s'"),
+			     _("Cannot find file '%s'"),
 			     basename);
 		return NULL;
 	}
@@ -1758,6 +1750,7 @@ ditem_execute (const GnomeDesktopItem *item,
 	char *exec_locale;
 	int launched = 0;
 #ifdef HAVE_STARTUP_NOTIFICATION
+	GdkDisplay *gdkdisplay;
 	SnLauncherContext *sn_context;
 	SnDisplay *sn_display;
 	const char *startup_class;
@@ -1794,7 +1787,12 @@ ditem_execute (const GnomeDesktopItem *item,
 	arg_ptr = make_args (file_list);
 
 #ifdef HAVE_STARTUP_NOTIFICATION
-	sn_display = sn_display_new (gdk_display,
+	if (screen)
+		gdkdisplay = gdk_screen_get_display (screen);
+	else
+		gdkdisplay = gdk_display_get_default ();
+
+	sn_display = sn_display_new (GDK_DISPLAY_XDISPLAY (gdkdisplay),
 				     sn_error_trap_push,
 				     sn_error_trap_pop);
 
@@ -1813,7 +1811,7 @@ ditem_execute (const GnomeDesktopItem *item,
 
 		sn_context = sn_launcher_context_new (sn_display,
 						      screen ? gdk_screen_get_number (screen) :
-						      DefaultScreen (gdk_display));
+						      DefaultScreen (GDK_DISPLAY_XDISPLAY (gdkdisplay)));
 		
 		name = gnome_desktop_item_get_localestring (item,
 							    GNOME_DESKTOP_ITEM_NAME);
@@ -1934,10 +1932,8 @@ ditem_execute (const GnomeDesktopItem *item,
 			
 			if (item->launch_time > 0)
 				launch_time = item->launch_time;
-			else if (screen != NULL)
-				launch_time = gdk_x11_display_get_user_time (gdk_screen_get_display (screen));
 			else
-				launch_time = gdk_x11_display_get_user_time (gdk_display_get_default ());
+				launch_time = gdk_x11_display_get_user_time (gdkdisplay);
 
 			sn_launcher_context_initiate (sn_context,
 						      g_get_prgname () ? g_get_prgname () : "unknown",
@@ -3526,8 +3522,11 @@ sanitize (GnomeDesktopItem *item, const char *uri)
 		if (name == NULL && uri != NULL)
 			name = g_path_get_basename (uri);
 		/* If no uri either, use same default as gnome_desktop_item_new */
-		if (name == NULL)
+		if (name == NULL) {
+		       /* Translators: the "name" mentioned here is the name of
+			* an application or a document */
 			name = g_strdup (_("No name"));
+		}
 		g_hash_table_replace (item->main_hash,
 				      g_strdup (GNOME_DESKTOP_ITEM_NAME), 
 				      name);
