@@ -31,12 +31,13 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 
+#include <X11/Xlib.h>
+#include <gdk/gdkx.h>
+
 #undef GNOME_DISABLE_DEPRECATED
 #include "libgnomeui/gnome-rr-config.h"
+
 #include "edid.h"
-#include <X11/Xlib.h>
-#include <X11/extensions/Xrandr.h>
-#include <gdk/gdkx.h>
 #include "gnome-rr-private.h"
 
 #define CONFIG_INTENDED_BASENAME "monitors.xml"
@@ -1406,34 +1407,79 @@ crtc_assignment_assign (CrtcAssignment   *assign,
 			int               y,
 			GnomeRRRotation   rotation,
                         gboolean          primary,
-			GnomeRROutput    *output)
+			GnomeRROutput    *output,
+			GError          **error)
 {
     CrtcInfo *info = g_hash_table_lookup (assign->info, crtc);
+    guint32 crtc_id;
+    const char *output_name;
 
-    if (!gnome_rr_crtc_can_drive_output (crtc, output) ||
-	!gnome_rr_output_supports_mode (output, mode)  ||
-	!gnome_rr_crtc_supports_rotation (crtc, rotation))
+    crtc_id = gnome_rr_crtc_get_id (crtc);
+    output_name = gnome_rr_output_get_name (output);
+
+    if (!gnome_rr_crtc_can_drive_output (crtc, output))
     {
+	g_set_error (error, GNOME_RR_ERROR, GNOME_RR_ERROR_CRTC_ASSIGNMENT,
+		     _("CRTC %d cannot drive output %s"), crtc_id, output_name);
+	return FALSE;
+    }
+
+    if (!gnome_rr_output_supports_mode (output, mode))
+    {
+	g_set_error (error, GNOME_RR_ERROR, GNOME_RR_ERROR_CRTC_ASSIGNMENT,
+		     _("output %s does not support mode %dx%d@%dHz"),
+		     output_name,
+		     gnome_rr_mode_get_width (mode),
+		     gnome_rr_mode_get_height (mode),
+		     gnome_rr_mode_get_freq (mode));
+	return FALSE;
+    }
+
+    if (!gnome_rr_crtc_supports_rotation (crtc, rotation))
+    {
+	g_set_error (error, GNOME_RR_ERROR, GNOME_RR_ERROR_CRTC_ASSIGNMENT,
+		     _("CRTC %d does not support rotation=%s"),
+		     crtc_id,
+		     get_rotation_name (rotation));
 	return FALSE;
     }
 
     if (info)
     {
-	if (info->mode == mode		&&
-	    info->x == x		&&
-	    info->y == y		&&
-	    info->rotation == rotation  &&
-	    can_clone (info, output))
+	if (!(info->mode == mode	&&
+	      info->x == x		&&
+	      info->y == y		&&
+	      info->rotation == rotation))
 	{
-	    g_ptr_array_add (info->outputs, output);
-
-            if (primary && !assign->primary)
-            {
-                assign->primary = output;
-            }
-
-	    return TRUE;
+	    g_set_error (error, GNOME_RR_ERROR, GNOME_RR_ERROR_CRTC_ASSIGNMENT,
+			 _("output %s does not have the same parameters as another cloned output:\n"
+			   "existing mode = %d, new mode = %d\n"
+			   "existing coordinates = (%d, %d), new coordinates = (%d, %d)\n"
+			   "existing rotation = %s, new rotation = %s"),
+			 output_name,
+			 gnome_rr_mode_get_id (info->mode), gnome_rr_mode_get_id (mode),
+			 info->x, info->y,
+			 x, y,
+			 get_rotation_name (info->rotation), get_rotation_name (rotation));
+	    return FALSE;
 	}
+
+	if (!can_clone (info, output))
+	{
+	    g_set_error (error, GNOME_RR_ERROR, GNOME_RR_ERROR_CRTC_ASSIGNMENT,
+			 _("cannot clone to output %s"),
+			 output_name);
+	    return FALSE;
+	}
+
+	g_ptr_array_add (info->outputs, output);
+
+	if (primary && !assign->primary)
+	{
+	    assign->primary = output;
+	}
+
+	return TRUE;
     }
     else
     {	
@@ -1456,8 +1502,6 @@ crtc_assignment_assign (CrtcAssignment   *assign,
 
 	return TRUE;
     }
-    
-    return FALSE;
 }
 
 static void
@@ -1543,6 +1587,13 @@ crtc_is_rotated (GnomeRRCrtc *crtc)
     return FALSE;
 }
 
+static void
+accumulate_error (GString *accumulated_error, GError *error)
+{
+    g_string_append_printf (accumulated_error, "    %s\n", error->message);
+    g_error_free (error);
+}
+
 /* Check whether the given set of settings can be used
  * at the same time -- ie. whether there is an assignment
  * of CRTC's to outputs.
@@ -1553,11 +1604,16 @@ crtc_is_rotated (GnomeRRCrtc *crtc)
 static gboolean
 real_assign_crtcs (GnomeRRScreen *screen,
 		   GnomeOutputInfo **outputs,
-		   CrtcAssignment *assignment)
+		   CrtcAssignment *assignment,
+		   GError **error)
 {
     GnomeRRCrtc **crtcs = gnome_rr_screen_list_crtcs (screen);
     GnomeOutputInfo *output;
     int i;
+    gboolean tried_mode;
+    GError *my_error;
+    GString *accumulated_error;
+    gboolean success;
 
     output = *outputs;
     if (!output)
@@ -1566,50 +1622,102 @@ real_assign_crtcs (GnomeRRScreen *screen,
     /* It is always allowed for an output to be turned off */
     if (!output->on)
     {
-	return real_assign_crtcs (screen, outputs + 1, assignment);
+	return real_assign_crtcs (screen, outputs + 1, assignment, error);
     }
 
+    success = FALSE;
+    tried_mode = FALSE;
+    accumulated_error = g_string_new (NULL);
+
     for (i = 0; crtcs[i] != NULL; ++i)
     {
+	GnomeRRCrtc *crtc = crtcs[i];
+	int crtc_id = gnome_rr_crtc_get_id (crtc);
 	int pass;
 
-	/* Make two passses, one where frequencies must match, then
+	g_string_append_printf (accumulated_error,
+				_("Trying modes for CRTC %d\n"),
+				crtc_id);
+
+	/* Make two passes, one where frequencies must match, then
 	 * one where they don't have to
 	 */
 	for (pass = 0; pass < 2; ++pass)
 	{
-	    GnomeRRCrtc *crtc = crtcs[i];
-	    GnomeRROutput *gnome_rr_output =
-		gnome_rr_screen_get_output_by_name (screen, output->name);
+	    GnomeRROutput *gnome_rr_output = gnome_rr_screen_get_output_by_name (screen, output->name);
 	    GnomeRRMode **modes = gnome_rr_output_list_modes (gnome_rr_output);
 	    int j;
-	
+
 	    for (j = 0; modes[j] != NULL; ++j)
 	    {
 		GnomeRRMode *mode = modes[j];
-		
-		if (gnome_rr_mode_get_width (mode) == output->width	&&
-		    gnome_rr_mode_get_height (mode) == output->height &&
-		    (pass == 1 || gnome_rr_mode_get_freq (mode) == output->rate))
+		int mode_width;
+		int mode_height;
+		int mode_freq;
+
+		mode_width = gnome_rr_mode_get_width (mode);
+		mode_height = gnome_rr_mode_get_height (mode);
+		mode_freq = gnome_rr_mode_get_freq (mode);
+
+		g_string_append_printf (accumulated_error,
+					_("CRTC %d: trying mode %dx%d@%dHz with output at %dx%d@%dHz (pass %d)\n"),
+					crtc_id,
+					mode_width, mode_height, mode_freq,
+					output->width, output->height, output->rate,
+					pass);
+
+		if (mode_width == output->width	&&
+		    mode_height == output->height &&
+		    (pass == 1 || mode_freq == output->rate))
 		{
+		    tried_mode = TRUE;
+
+		    my_error = NULL;
 		    if (crtc_assignment_assign (
 			    assignment, crtc, modes[j],
 			    output->x, output->y,
 			    output->rotation,
                             output->primary,
-			    gnome_rr_output))
+			    gnome_rr_output,
+			    &my_error))
 		    {
-			if (real_assign_crtcs (screen, outputs + 1, assignment))
-			    return TRUE;
-			
+			my_error = NULL;
+			if (real_assign_crtcs (screen, outputs + 1, assignment, &my_error)) {
+			    success = TRUE;
+			    goto out;
+			} else
+			    accumulate_error (accumulated_error, my_error);
+
 			crtc_assignment_unassign (assignment, crtc, gnome_rr_output);
-		    }
+		    } else
+			accumulate_error (accumulated_error, my_error);
 		}
 	    }
 	}
     }
 
-    return FALSE;
+out:
+
+    if (success)
+	g_string_free (accumulated_error, TRUE);
+    else {
+	char *str;
+
+	str = g_string_free (accumulated_error, FALSE);
+
+	if (tried_mode)
+	    g_set_error (error, GNOME_RR_ERROR, GNOME_RR_ERROR_CRTC_ASSIGNMENT,
+			 _("could not assign CRTCs to outputs:\n%s"),
+			 str);
+	else
+	    g_set_error (error, GNOME_RR_ERROR, GNOME_RR_ERROR_CRTC_ASSIGNMENT,
+			 _("none of the selected modes were compatible with the possible modes:\n%s"),
+			 str);
+
+	g_free (str);
+    }
+
+    return success;
 }
 
 static void
@@ -1664,7 +1772,7 @@ crtc_assignment_new (GnomeRRScreen *screen, GnomeOutputInfo **outputs, GError **
     assignment->info = g_hash_table_new_full (
 	g_direct_hash, g_direct_equal, NULL, (GFreeFunc)crtc_info_free);
 
-    if (real_assign_crtcs (screen, outputs, assignment))
+    if (real_assign_crtcs (screen, outputs, assignment, error))
     {
 	int width, height;
 	int min_width, max_width, min_height, max_height;
@@ -1697,10 +1805,7 @@ crtc_assignment_new (GnomeRRScreen *screen, GnomeOutputInfo **outputs, GError **
 	
 	return assignment;
     }
-    else
-	g_set_error (error, GNOME_RR_ERROR, GNOME_RR_ERROR_CRTC_ASSIGNMENT,
-		     _("could not find a suitable configuration of screens"));
-    
+
 fail:
     crtc_assignment_free (assignment);
     
