@@ -26,19 +26,39 @@
 
 #include <config.h>
 #include <glib/gi18n-lib.h>
-#include "libgnomeui/gnome-rr.h"
 #include <string.h>
 #include <X11/Xlib.h>
+
+#ifdef HAVE_RANDR
 #include <X11/extensions/Xrandr.h>
+#endif
+
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <X11/Xatom.h>
 
+#undef GNOME_DISABLE_DEPRECATED
+#include "libgnomeui/gnome-rr.h"
+
 #include "private.h"
 #include "gnome-rr-private.h"
 
 #define DISPLAY(o) ((o)->info->screen->xdisplay)
 
+#ifndef HAVE_RANDR
+/* This is to avoid a ton of ifdefs wherever we use a type from libXrandr */
+typedef int RROutput;
+typedef int RRCrtc;
+typedef int RRMode;
+typedef int Rotation;
+#define RR_Rotate_0		1
+#define RR_Rotate_90		2
+#define RR_Rotate_180		4
+#define RR_Rotate_270		8
+#define RR_Reflect_X		16
+#define RR_Reflect_Y		32
+#endif
+
 struct GnomeRROutput
 {
     ScreenInfo *	info;
@@ -54,6 +74,7 @@ struct GnomeRROutput
     GnomeRRMode **	modes;
     int			n_preferred;
     guint8 *		edid_data;
+    char *              connector_type;
 };
 
 struct GnomeRROutputWrap
@@ -91,23 +112,34 @@ struct GnomeRRMode
 static GnomeRRCrtc *  crtc_new          (ScreenInfo         *info,
 					 RRCrtc              id);
 static void           crtc_free         (GnomeRRCrtc        *crtc);
+
+#ifdef HAVE_RANDR
 static gboolean       crtc_initialize   (GnomeRRCrtc        *crtc,
 					 XRRScreenResources *res,
 					 GError            **error);
+#endif
 
 /* GnomeRROutput */
 static GnomeRROutput *output_new        (ScreenInfo         *info,
 					 RROutput            id);
+
+#ifdef HAVE_RANDR
 static gboolean       output_initialize (GnomeRROutput      *output,
 					 XRRScreenResources *res,
 					 GError            **error);
+#endif
+
 static void           output_free       (GnomeRROutput      *output);
 
 /* GnomeRRMode */
 static GnomeRRMode *  mode_new          (ScreenInfo         *info,
 					 RRMode              id);
+
+#ifdef HAVE_RANDR
 static void           mode_initialize   (GnomeRRMode        *mode,
 					 XRRModeInfo        *info);
+#endif
+
 static void           mode_free         (GnomeRRMode        *mode);
 
 
@@ -185,13 +217,15 @@ screen_info_free (ScreenInfo *info)
     GnomeRRMode **mode;
     
     g_assert (info != NULL);
-    
+
+#ifdef HAVE_RANDR
     if (info->resources)
     {
 	XRRFreeScreenResources (info->resources);
 	
 	info->resources = NULL;
     }
+#endif
     
     if (info->outputs)
     {
@@ -292,6 +326,7 @@ gather_clone_modes (ScreenInfo *info)
     info->clone_modes = (GnomeRRMode **)g_ptr_array_free (result, FALSE);
 }
 
+#ifdef HAVE_RANDR
 static gboolean
 fill_screen_info_from_resources (ScreenInfo *info,
 				 XRRScreenResources *resources,
@@ -361,6 +396,7 @@ fill_screen_info_from_resources (ScreenInfo *info,
 
     return TRUE;
 }
+#endif /* HAVE_RANDR */
 
 static gboolean
 fill_out_screen_info (Display *xdisplay,
@@ -369,6 +405,7 @@ fill_out_screen_info (Display *xdisplay,
 		      gboolean needs_reprobe,
 		      GError **error)
 {
+#ifdef HAVE_RANDR
     XRRScreenResources *resources;
     
     g_assert (xdisplay != NULL);
@@ -402,8 +439,8 @@ fill_out_screen_info (Display *xdisplay,
     }
     else
     {
-        /* Translators: a CRTC is a CRT Controller (this is X terminology). */
 	g_set_error (error, GNOME_RR_ERROR, GNOME_RR_ERROR_RANDR_ERROR,
+		     /* Translators: a CRTC is a CRT Controller (this is X terminology). */
 		     _("could not get the screen resources (CRTCs, outputs, modes)"));
 	return FALSE;
     }
@@ -455,6 +492,9 @@ fill_out_screen_info (Display *xdisplay,
 #endif
 
     return TRUE;
+#else
+    return FALSE;
+#endif /* HAVE_RANDR */
 }
 
 static ScreenInfo *
@@ -492,9 +532,11 @@ screen_update (GnomeRRScreen *screen, gboolean force_callback, gboolean needs_re
     if (!info)
 	    return FALSE;
 
+#ifdef HAVE_RANDR
     if (info->resources->configTimestamp != screen->info->resources->configTimestamp)
 	    changed = TRUE;
-	
+#endif
+
     screen_info_free (screen->info);
 	
     screen->info = info;
@@ -510,6 +552,7 @@ screen_on_event (GdkXEvent *xevent,
 		 GdkEvent *event,
 		 gpointer data)
 {
+#ifdef HAVE_RANDR
     GnomeRRScreen *screen = data;
     XEvent *e = xevent;
     int event_num;
@@ -594,6 +637,8 @@ screen_on_event (GdkXEvent *xevent,
     }
 #endif
 
+#endif /* HAVE_RANDR */
+
     /* Pass the event on to GTK+ */
     return GDK_FILTER_CONTINUE;
 }
@@ -607,14 +652,17 @@ gnome_rr_screen_new (GdkScreen *gdk_screen,
 		     gpointer data,
 		     GError **error)
 {
+#ifdef HAVE_RANDR
     Display *dpy = GDK_SCREEN_XDISPLAY (gdk_screen);
     int event_base;
     int ignore;
+#endif
 
     g_return_val_if_fail (error == NULL || *error == NULL, NULL);
     
     _gnome_desktop_init_i18n ();
 
+#ifdef HAVE_RANDR
     if (XRRQueryExtension (dpy, &event_base, &ignore))
     {
 	GnomeRRScreen *screen = g_new0 (GnomeRRScreen, 1);
@@ -624,6 +672,7 @@ gnome_rr_screen_new (GdkScreen *gdk_screen,
 	screen->xroot = gdk_x11_drawable_get_xid (screen->gdk_root);
 	screen->xdisplay = dpy;
 	screen->xscreen = gdk_x11_screen_get_xscreen (screen->gdk_screen);
+	screen->connector_type_atom = XInternAtom (dpy, "ConnectorType", FALSE);
 	
 	screen->callback = callback;
 	screen->data = data;
@@ -631,6 +680,12 @@ gnome_rr_screen_new (GdkScreen *gdk_screen,
 	screen->randr_event_base = event_base;
 
 	XRRQueryVersion (dpy, &screen->rr_major_version, &screen->rr_minor_version);
+	if (screen->rr_major_version > 1 || (screen->rr_major_version == 1 && screen->rr_minor_version < 2)) {
+	    g_set_error (error, GNOME_RR_ERROR, GNOME_RR_ERROR_NO_RANDR_EXTENSION,
+			 "RANDR extension is too old (must be at least 1.2)");
+	    g_free (screen);
+	    return NULL;
+	}
 
 	screen->info = screen_info_new (screen, TRUE, error);
 	
@@ -655,11 +710,14 @@ gnome_rr_screen_new (GdkScreen *gdk_screen,
     }
     else
     {
+#endif /* HAVE_RANDR */
 	g_set_error (error, GNOME_RR_ERROR, GNOME_RR_ERROR_NO_RANDR_EXTENSION,
 		     _("RANDR extension is not present"));
 	
 	return NULL;
+#ifdef HAVE_RANDR
    }
+#endif
 }
 
 void
@@ -683,9 +741,14 @@ gnome_rr_screen_set_size (GnomeRRScreen *screen,
 			  int       mm_height)
 {
     g_return_if_fail (screen != NULL);
-    
+
+#ifdef HAVE_RANDR
+    gdk_error_trap_push ();
     XRRSetScreenSize (screen->xdisplay, screen->xroot,
 		      width, height, mm_width, mm_height);
+    gdk_flush ();
+    gdk_error_trap_pop (); /* ignore error */
+#endif
 }
 
 void
@@ -729,13 +792,60 @@ gnome_rr_screen_get_timestamps (GnomeRRScreen *screen,
 {
     g_return_if_fail (screen != NULL);
 
+#ifdef HAVE_RANDR
     if (change_timestamp_ret)
 	*change_timestamp_ret = screen->info->resources->timestamp;
 
     if (config_timestamp_ret)
 	*config_timestamp_ret = screen->info->resources->configTimestamp;
+#endif
 }
 
+static gboolean
+force_timestamp_update (GnomeRRScreen *screen)
+{
+    GnomeRRCrtc *crtc;
+    XRRCrtcInfo *current_info;
+    Status status;
+    gboolean timestamp_updated;
+
+    timestamp_updated = FALSE;
+
+    crtc = screen->info->crtcs[0];
+
+    if (crtc == NULL)
+	goto out;
+
+    current_info = XRRGetCrtcInfo (screen->xdisplay,
+				   screen->info->resources,
+				   crtc->id);
+
+    if (current_info == NULL)
+	goto out;
+
+    gdk_error_trap_push ();
+    status = XRRSetCrtcConfig (screen->xdisplay,
+			       screen->info->resources,
+			       crtc->id,
+			       current_info->timestamp,
+			       current_info->x,
+			       current_info->y,
+			       current_info->mode,
+			       current_info->rotation,
+			       current_info->outputs,
+			       current_info->noutput);
+
+    XRRFreeCrtcInfo (current_info);
+
+    gdk_flush ();
+    if (gdk_error_trap_pop ())
+	goto out;
+
+    if (status == RRSetConfigSuccess)
+	timestamp_updated = TRUE;
+out:
+    return timestamp_updated;
+}
 
 /**
  * gnome_rr_screen_refresh
@@ -754,8 +864,18 @@ gboolean
 gnome_rr_screen_refresh (GnomeRRScreen *screen,
 			 GError       **error)
 {
+    gboolean refreshed;
+
     g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
-    return screen_update (screen, FALSE, TRUE, error);
+
+    gdk_x11_display_grab (gdk_screen_get_display (screen->gdk_screen));
+
+    refreshed = screen_update (screen, FALSE, TRUE, error);
+    force_timestamp_update (screen); /* this is to keep other clients from thinking that the X server re-detected things by itself - bgo#621046 */
+
+    gdk_x11_display_ungrab (gdk_screen_get_display (screen->gdk_screen));
+
+    return refreshed;
 }
 
 GnomeRRMode **
@@ -848,6 +968,7 @@ get_property (Display *dpy,
 	      Atom atom,
 	      int *len)
 {
+#ifdef HAVE_RANDR
     unsigned char *prop;
     int actual_format;
     unsigned long nitems, bytes_after;
@@ -874,6 +995,9 @@ get_property (Display *dpy,
     XFree (prop);
     
     return result;
+#else
+    return NULL;
+#endif /* HAVE_RANDR */
 }
 
 static guint8 *
@@ -905,6 +1029,49 @@ read_edid_data (GnomeRROutput *output)
     return NULL;
 }
 
+static char *
+get_connector_type_string (GnomeRROutput *output)
+{
+#ifdef HAVE_RANDR
+    char *result;
+    unsigned char *prop;
+    int actual_format;
+    unsigned long nitems, bytes_after;
+    Atom actual_type;
+    Atom connector_type;
+    char *connector_type_str;
+
+    result = NULL;
+
+    if (XRRGetOutputProperty (DISPLAY (output), output->id, output->info->screen->connector_type_atom,
+			      0, 100, False, False,
+			      AnyPropertyType,
+			      &actual_type, &actual_format,
+			      &nitems, &bytes_after, &prop) != Success)
+	return NULL;
+
+    if (!(actual_type == XA_ATOM && actual_format == 32 && nitems == 1))
+	goto out;
+
+    connector_type = *((Atom *) prop);
+
+    connector_type_str = XGetAtomName (DISPLAY (output), connector_type);
+    if (connector_type_str) {
+	result = g_strdup (connector_type_str); /* so the caller can g_free() it */
+	XFree (connector_type_str);
+    }
+
+out:
+
+    XFree (prop);
+
+    return result;
+#else
+    return NULL;
+#endif
+}
+
+#ifdef HAVE_RANDR
 static gboolean
 output_initialize (GnomeRROutput *output, XRRScreenResources *res, GError **error)
 {
@@ -932,7 +1099,8 @@ output_initialize (GnomeRROutput *output, XRRScreenResources *res, GError **erro
     output->width_mm = info->mm_width;
     output->height_mm = info->mm_height;
     output->connected = (info->connection == RR_Connected);
-    
+    output->connector_type = get_connector_type_string (output);
+
     /* Possible crtcs */
     a = g_ptr_array_new ();
     
@@ -979,6 +1147,7 @@ output_initialize (GnomeRROutput *output, XRRScreenResources *res, GError **erro
 
     return TRUE;
 }
+#endif /* HAVE_RANDR */
 
 static void
 output_free (GnomeRROutput *output)
@@ -988,6 +1157,7 @@ output_free (GnomeRROutput *output)
     g_free (output->possible_crtcs);
     g_free (output->edid_data);
     g_free (output->name);
+    g_free (output->connector_type);
     g_free (output);
 }
 
@@ -1035,6 +1205,45 @@ gnome_rr_output_get_crtc (GnomeRROutput *output)
     return output->current_crtc;
 }
 
+/* Returns NULL if the ConnectorType property is not available */
+const char *
+gnome_rr_output_get_connector_type (GnomeRROutput *output)
+{
+    g_return_val_if_fail (output != NULL, NULL);
+
+    return output->connector_type;
+}
+
+gboolean
+gnome_rr_output_is_laptop (GnomeRROutput *output)
+{
+    const char *connector_type;
+
+    g_return_val_if_fail (output != NULL, FALSE);
+
+    if (!output->connected)
+	return FALSE;
+
+    /* The ConnectorType property is present in RANDR 1.3 and greater */
+
+    connector_type = gnome_rr_output_get_connector_type (output);
+    if (connector_type && strcmp (connector_type, GNOME_RR_CONNECTOR_TYPE_PANEL) == 0)
+	return TRUE;
+
+    /* Older versions of RANDR - this is a best guess, as @#$% RANDR doesn't have standard output names,
+     * so drivers can use whatever they like.
+     */
+
+    if (output->name
+	&& (strstr (output->name, "lvds") ||  /* Most drivers use an "LVDS" prefix... */
+	    strstr (output->name, "LVDS") ||
+	    strstr (output->name, "Lvds") ||
+	    strstr (output->name, "LCD")))    /* ... but fglrx uses "LCD" in some versions.  Shoot me now, kthxbye. */
+	return TRUE;
+
+    return FALSE;
+}
+
 GnomeRRMode *
 gnome_rr_output_get_current_mode (GnomeRROutput *output)
 {
@@ -1145,13 +1354,18 @@ gnome_rr_output_can_clone (GnomeRROutput *output,
 gboolean
 gnome_rr_output_get_is_primary (GnomeRROutput *output)
 {
+#ifdef HAVE_RANDR
     return output->info->primary == output->id;
+#else
+    return FALSE;
+#endif
 }
 
 void
 gnome_rr_screen_set_primary_output (GnomeRRScreen *screen,
                                     GnomeRROutput *output)
 {
+#ifdef HAVE_RANDR
 #if (RANDR_MAJOR > 1 || (RANDR_MAJOR == 1 && RANDR_MINOR >= 3))
     RROutput id;
 
@@ -1164,6 +1378,7 @@ gnome_rr_screen_set_primary_output (GnomeRRScreen *screen,
     if (screen->rr_major_version == 1 && screen->rr_minor_version >= 3)
         XRRSetOutputPrimary (screen->xdisplay, screen->xroot, id);
 #endif
+#endif /* HAVE_RANDR */
 }
 
 /* GnomeRRCrtc */
@@ -1239,6 +1454,7 @@ gnome_rr_crtc_set_config_with_time (GnomeRRCrtc      *crtc,
 				    int               n_outputs,
 				    GError          **error)
 {
+#ifdef HAVE_RANDR
     ScreenInfo *info;
     GArray *output_ids;
     Status status;
@@ -1301,6 +1517,9 @@ gnome_rr_crtc_set_config_with_time (GnomeRRCrtc      *crtc,
     }
     
     return result;
+#else
+    return FALSE;
+#endif /* HAVE_RANDR */
 }
 
 GnomeRRMode *
@@ -1386,6 +1605,7 @@ crtc_new (ScreenInfo *info, RROutput id)
     return crtc;
 }
 
+#ifdef HAVE_RANDR
 static gboolean
 crtc_initialize (GnomeRRCrtc        *crtc,
 		 XRRScreenResources *res,
@@ -1454,6 +1674,7 @@ crtc_initialize (GnomeRRCrtc        *crtc,
 
     return TRUE;
 }
+#endif
 
 static void
 crtc_free (GnomeRRCrtc *crtc)
@@ -1503,6 +1724,7 @@ gnome_rr_mode_get_height (GnomeRRMode *mode)
     return mode->height;
 }
 
+#ifdef HAVE_RANDR
 static void
 mode_initialize (GnomeRRMode *mode, XRRModeInfo *info)
 {
@@ -1514,6 +1736,7 @@ mode_initialize (GnomeRRMode *mode, XRRModeInfo *info)
     mode->height = info->height;
     mode->freq = ((info->dotClock / (double)info->hTotal) / info->vTotal + 0.5) * 1000;
 }
+#endif /* HAVE_RANDR */
 
 static void
 mode_free (GnomeRRMode *mode)
@@ -1528,6 +1751,7 @@ gnome_rr_crtc_set_gamma (GnomeRRCrtc *crtc, int size,
 			 unsigned short *green,
 			 unsigned short *blue)
 {
+#ifdef HAVE_RANDR
     int copy_size;
     XRRCrtcGamma *gamma;
 
@@ -1548,6 +1772,7 @@ gnome_rr_crtc_set_gamma (GnomeRRCrtc *crtc, int size,
 
     XRRSetCrtcGamma (DISPLAY (crtc), crtc->id, gamma);
     XRRFreeGamma (gamma);
+#endif /* HAVE_RANDR */
 }
 
 gboolean
@@ -1555,6 +1780,7 @@ gnome_rr_crtc_get_gamma (GnomeRRCrtc *crtc, int *size,
 			 unsigned short **red, unsigned short **green,
 			 unsigned short **blue)
 {
+#ifdef HAVE_RANDR
     int copy_size;
     unsigned short *r, *g, *b;
     XRRCrtcGamma *gamma;
@@ -1591,5 +1817,8 @@ gnome_rr_crtc_get_gamma (GnomeRRCrtc *crtc, int *size,
 	*size = crtc->gamma_size;
 
     return TRUE;
+#else
+    return FALSE;
+#endif /* HAVE_RANDR */
 }
 
