@@ -59,7 +59,8 @@ typedef enum {
 	GNOME_BG_PLACEMENT_ZOOMED,
 	GNOME_BG_PLACEMENT_CENTERED,
 	GNOME_BG_PLACEMENT_SCALED,
-	GNOME_BG_PLACEMENT_FILL_SCREEN
+	GNOME_BG_PLACEMENT_FILL_SCREEN,
+	GNOME_BG_PLACEMENT_SPANNED
 } GnomeBGPlacement;
 
 GType            gnome_bg_get_type              (void);
@@ -87,7 +88,9 @@ const gchar *    gnome_bg_get_filename          (GnomeBG               *bg);
 
 /* Drawing and thumbnailing */
 void             gnome_bg_draw                  (GnomeBG               *bg,
-						 GdkPixbuf             *dest);
+						 GdkPixbuf             *dest,
+						 GdkScreen	       *screen,
+                                                 gboolean               is_root);
 GdkPixmap *      gnome_bg_create_pixmap         (GnomeBG               *bg,
 						 GdkWindow             *window,
 						 int                    width,
@@ -95,6 +98,8 @@ GdkPixmap *      gnome_bg_create_pixmap         (GnomeBG               *bg,
 						 gboolean               root);
 gboolean         gnome_bg_get_image_size        (GnomeBG               *bg,
 						 GnomeDesktopThumbnailFactory *factory,
+                                                 int                    best_width,
+                                                 int                    best_height,
 						 int                   *width,
 						 int                   *height);
 GdkPixbuf *      gnome_bg_create_thumbnail      (GnomeBG               *bg,
@@ -102,8 +107,10 @@ GdkPixbuf *      gnome_bg_create_thumbnail      (GnomeBG               *bg,
 						 GdkScreen             *screen,
 						 int                    dest_width,
 						 int                    dest_height);
-gboolean         gnome_bg_is_dark               (GnomeBG               *bg);
-gboolean         gnome_bg_changes_with_size     (GnomeBG               *bg);
+gboolean         gnome_bg_is_dark               (GnomeBG               *bg,
+                                                 int                    dest_width,
+						 int                    dest_height);
+gboolean         gnome_bg_has_multiple_sizes    (GnomeBG               *bg);
 gboolean         gnome_bg_changes_with_time     (GnomeBG               *bg);
 GdkPixbuf *      gnome_bg_create_frame_thumbnail (GnomeBG              *bg,
 						 GnomeDesktopThumbnailFactory *factory,
